@@ -47,7 +47,8 @@ void ssEAP::setup(SEXP robj, double qwidth, int qpts, int *_mask)
 	mask = _mask;
 
 	grp.setGridFineness(qwidth, qpts);
-	grp.import(robj);
+	grp.import(robj, true);
+	grp.setupQuadrature();
 }
 
 void ssEAP::setLastItem(int which)
@@ -103,7 +104,7 @@ void ssEAP::tpbw1995Vanilla()
 			for (int dx=0; dx < dims; dx++) {
 				ptheta[dx] = where[std::min(dx, quad.maxDims-1)];
 			}
-			librpf_model[id].prob(spec, iparam, ptheta.data(), oprob.data());
+			Glibrpf_model[id].prob(spec, iparam, ptheta.data(), oprob.data());
 			for (int ox=0; ox < outcomes; ++ox) {
 				slCur(qx, ox) *= oprob[ox];
 			}
@@ -129,7 +130,7 @@ void ssEAP::tpbw1995Vanilla()
 			for (int dx=0; dx < dims; dx++) {
 				ptheta[dx] = where[std::min(dx, quad.maxDims-1)];
 			}
-			librpf_model[id].prob(spec, iparam, ptheta.data(), oprob.data());
+			Glibrpf_model[id].prob(spec, iparam, ptheta.data(), oprob.data());
 			for (int cx=0; cx <= curMax; cx++) {
 				for (int ox=0; ox < outcomes; ox++) {
 					slCur(qx, cx + ox) += slPrev(qx, cx) * oprob[ox];
@@ -264,7 +265,7 @@ void ssEAP::tpbw1995TwoTier()
 			for (int dx=0; dx < dims; dx++) {
 				ptheta[dx] = where[std::min(dx, quad.maxDims-1)];
 			}
-			librpf_model[id].prob(spec, iparam, ptheta.data(), oprob.data());
+			Glibrpf_model[id].prob(spec, iparam, ptheta.data(), oprob.data());
 			for (int ox=0; ox < outcomes; ++ox) {
 				ttCur(qx * quad.numSpecific + Sgroup, ox) *= oprob[ox];
 			}
@@ -291,7 +292,7 @@ void ssEAP::tpbw1995TwoTier()
 			for (int dx=0; dx < dims; dx++) {
 				ptheta[dx] = where[std::min(dx, quad.maxDims-1)];
 			}
-			librpf_model[id].prob(spec, iparam, ptheta.data(), oprob.data());
+			Glibrpf_model[id].prob(spec, iparam, ptheta.data(), oprob.data());
 			for (int cx=0; cx <= ttCurMax(Sgroup); cx++) {
 				for (int ox=0; ox < outcomes; ox++) {
 					ttCur(row, cx + ox) += ttPrev(row, cx) * oprob[ox];
@@ -361,7 +362,7 @@ SEXP ot2000_wrapper(SEXP robj, SEXP Ritem, SEXP Rwidth, SEXP Rpts, SEXP Ralter,
 			for (int dx=0; dx < dims; dx++) {
 				ptheta[dx] = where[std::min(dx, quad.maxDims-1)];
 			}
-			librpf_model[id].prob(spec, iparam, ptheta.data(), oprob.data());
+			Glibrpf_model[id].prob(spec, iparam, ptheta.data(), oprob.data());
 			for (int ox=0; ox < outcomes; ox++) {
 				iProb(qx, ox) = oprob[ox];
 			}
@@ -494,7 +495,8 @@ SEXP pairwiseExpected(SEXP robj, SEXP Rwidth, SEXP Rpts, SEXP Ritems, SEXP Rtwot
 
 	ifaGroup grp(1, Rf_asLogical(Rtwotier));
 	grp.setGridFineness(Rf_asReal(Rwidth), Rf_asInteger(Rpts));
-	grp.import(robj);
+	grp.import(robj, false); // lenient=true is probably okay, need to test
+	grp.setupQuadrature();
 	
 	ba81NormalQuad &quad = grp.quad;
 
@@ -544,8 +546,8 @@ SEXP pairwiseExpected(SEXP robj, SEXP Rwidth, SEXP Rpts, SEXP Ritems, SEXP Rtwot
 		int specificIncr = quad.numSpecific? quad.quadGridSize : 1;
 		for (int qx=0; qx < quad.totalPrimaryPoints; ++qx) {
 			double *where = quad.wherePrep.data() + qx * quad.maxDims * specificIncr;
-			(*librpf_model[id1].prob)(spec1, i1par, where, o1.data());
-			(*librpf_model[id2].prob)(spec2, i2par, where, o2.data());
+			(*Glibrpf_model[id1].prob)(spec1, i1par, where, o1.data());
+			(*Glibrpf_model[id2].prob)(spec2, i2par, where, o2.data());
 			out += (o1 * o2.transpose()) * quad.priQarea[qx];
 		}
 	} else if (specific1 == specific2) {
@@ -556,8 +558,8 @@ SEXP pairwiseExpected(SEXP robj, SEXP Rwidth, SEXP Rpts, SEXP Ritems, SEXP Rtwot
 				for (int dx=0; dx < quad.maxAbilities; dx++) {
 					ptheta[dx] = where[std::min(dx, quad.maxDims-1)];
 				}
-				(*librpf_model[id1].prob)(spec1, i1par, ptheta.data(), o1.data());
-				(*librpf_model[id2].prob)(spec2, i2par, ptheta.data(), o2.data());
+				(*Glibrpf_model[id1].prob)(spec1, i1par, ptheta.data(), o1.data());
+				(*Glibrpf_model[id2].prob)(spec2, i2par, ptheta.data(), o2.data());
 				double area = quad.priQarea[qx] * quad.speQarea[sx * quad.numSpecific + specific1];
 				out += (o1 * o2.transpose()) * area;
 				++qloc;
@@ -575,8 +577,8 @@ SEXP pairwiseExpected(SEXP robj, SEXP Rwidth, SEXP Rpts, SEXP Ritems, SEXP Rtwot
 				for (int dx=0; dx < quad.maxAbilities; dx++) {
 					ptheta[dx] = where[std::min(dx, quad.maxDims-1)];
 				}
-				(*librpf_model[id1].prob)(spec1, i1par, ptheta.data(), spo1.data());
-				(*librpf_model[id2].prob)(spec2, i2par, ptheta.data(), spo2.data());
+				(*Glibrpf_model[id1].prob)(spec1, i1par, ptheta.data(), spo1.data());
+				(*Glibrpf_model[id2].prob)(spec2, i2par, ptheta.data(), spo2.data());
 				if (specific1 == -1) {
 					if (sx==0) o1 = spo1;
 				} else {
@@ -761,7 +763,7 @@ SEXP observedSumScore(SEXP Rgrp, SEXP Rmask)
 	omxManageProtectInsanity mpi;
 
 	ifaGroup grp(1, false);
-	grp.import(Rgrp);
+	grp.import(Rgrp, true);
 	if (grp.getNumUnique() == 0) Rf_error("observedSumScore requires data");
 
 	if (Rf_length(Rmask) != int(grp.spec.size())) {
@@ -796,7 +798,7 @@ SEXP itemOutcomeBySumScore(SEXP Rgrp, SEXP Rmask, SEXP Rinterest)
 	omxManageProtectInsanity mpi;
 
 	ifaGroup grp(1, false);
-	grp.import(Rgrp);
+	grp.import(Rgrp, true);
 	if (grp.getNumUnique() == 0) Rf_error("itemOutcomeBySumScore requires data");
 
 	if (Rf_length(Rmask) != int(grp.spec.size())) {
